/*
 * MusicPOV.cpp
 *
 * Created: 5/18/2019 2:32:37 PM
 * Author : Cornel
 */ 

#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#include <stdlib.h>

#include "ADC.h"

#define MIC_AMP_FACTOR 100
#define MIC_ADC_CHANNEL 7
#define TH 400

void Light_R(char* mode)
{
	if(mode[0])
		PORTA |= (1 << PORTA0);
	if(mode[1])
		PORTA |= (1 << PORTA3);
	if(mode[2])
		PORTC |= (1 << PORTC5);
	if(mode[3])
		PORTD |= (1 << PORTD0);
	if(mode[4])
		PORTB |= (1 << PORTB3);
}

void Light_G(char* mode)
{
	if(mode[0])
		PORTA |= (1 << PORTA1);
	if(mode[1])
		PORTA |= (1 << PORTA4);
	if(mode[2])
		PORTC |= (1 << PORTC4);
	if(mode[3])
		PORTD |= (1 << PORTD1);
	if(mode[4])
		PORTB |= (1 << PORTB4);
}

void Light_B(char* mode)
{
	if(mode[0])
		PORTA |= (1 << PORTA2);
	if(mode[1])
		PORTA |= (1 << PORTA5);
	if(mode[2])
		PORTC |= (1 << PORTC3);
	if(mode[3])
		PORTD |= (1 << PORTD6);
	if(mode[4])
		PORTB |= (1 << PORTB5);
}

void Light_off()
{
		PORTA &= ~(1 << PORTA0) & ~(1 << PORTA1) & ~(1 << PORTA2);
		PORTA &= ~(1 << PORTA3) & ~(1 << PORTA4) & ~(1 << PORTA5);
		PORTC &= ~(1 << PORTC5) & ~(1 << PORTC4) & ~(1 << PORTC3);
		PORTD &= ~(1 << PORTD0) & ~(1 << PORTD1) & ~(1 << PORTD6);
		PORTB &= ~(1 << PORTB3) & ~(1 << PORTB4) & ~(1 << PORTB5);
}

void Random_set_array(char* array, uint8_t size)
{
	uint8_t i;
	uint8_t val;
	for(i = 0; i < size; i++)
	{
		val = rand() % 9 + 1;
		array[i] = val % 2 == 0? 1 : 0;
	}
	
	uint8_t min_set_values = size / 2;
	uint8_t set_values = 0;
	
	for(i = 0; i < size; i++)
	{
		if(array[i])
			set_values++;
			
		if(set_values == min_set_values)
			return;
	}
	
	Random_set_array(array, size);
}

void Light(char* l_mode, char* r_mode, char* g_mode, char* b_mode)
{
	if(l_mode[0])
		Light_R(r_mode);
	if(l_mode[1])
		Light_G(r_mode);
	if(l_mode[2])
		Light_B(b_mode);
}

void Init_LEDs()
{
	// Set LEDs as output
	DDRA |= (1 << PORTA0) | (1 << PORTA1) | (1 << PORTA2); // LED 1
	DDRA |= (1 << PORTA3) | (1 << PORTA4) | (1 << PORTA5); // LED 2
	DDRC |= (1 << PORTC5) | (1 << PORTC4) | (1 << PORTC3); // LED 3
	DDRD |= (1 << PORTD0) | (1 << PORTD1) | (1 << PORTD6); // LED 4
	DDRB |= (1 << PORTB3) | (1 << PORTB4) | (1 << PORTB5); // LED 5	
	
	Light_off();
}

void Init_Mic()
{
	ADC_init();
}

double Get_Data_Mic()
{
	return ADC_voltage(ADC_get(MIC_ADC_CHANNEL)) * MIC_AMP_FACTOR;
}

int main(void)
{
	Init_LEDs();
	Init_Mic();
	
	const uint8_t N_COLORS = 3;
	const uint8_t N_LEDS = 5;
	const uint8_t MAX_ITER = 40;
	
	int iter = 0;
	double value;
	
	char l_mode[N_LEDS];
	char r_mode[N_COLORS];
	char g_mode[N_COLORS];
	char b_mode[N_COLORS];
	
	Random_set_array(l_mode, N_LEDS);
	Random_set_array(r_mode, N_COLORS);
	Random_set_array(g_mode, N_COLORS);
	Random_set_array(b_mode, N_COLORS);
	
    while (1) 
	{
		Light_off();
	    
		value = Get_Data_Mic();
		if(value > TH)
		{
			Light(l_mode, r_mode, g_mode, b_mode);
			iter += value * 2 / TH;
			if(iter >= MAX_ITER){
				Random_set_array(l_mode, N_LEDS);
				Random_set_array(r_mode, N_COLORS);
				Random_set_array(g_mode, N_COLORS);
				Random_set_array(b_mode, N_COLORS);
				
				iter = 0;			
			}
			_delay_ms(20);
		}
    }
}

